"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const getPort = require("get-port");
const sentryApp = require("probot/lib/apps/sentry");
const logRequestErrors = require("probot/lib/middleware/log-request-errors");
const landing_page_1 = require("./landing-page");
const utils_1 = require("./utils");
const getBody = (request) => new Promise((resolve, reject) => {
    const chunks = [];
    request.on("data", chunk => {
        chunks.push(chunk);
    });
    request.on("end", () => {
        const body = Buffer.concat(chunks).toString();
        resolve(body);
    });
    request.on("error", reject);
});
const initializeProbot = (applicationFn) => {
    const probot = utils_1.createProbot();
    // Log all unhandled rejections
    process.on("unhandledRejection", probot.errorHandler);
    const application = probot.load(applicationFn);
    probot.load(sentryApp);
    probot.server.use(logRequestErrors);
    application
        .route("/")
        .get("/", async (internalRequest, internalResponse) => {
        const name = await utils_1.fetchAppName(application);
        internalResponse.statusCode = 200;
        internalResponse.end(landing_page_1.getLandingPage(name));
    });
    return probot;
};
/**
 * Wrap the given application function as a lambda.
 *
 * @param applicationFn - The function corresponding to this GitHub App.
 */
const toLambda = (applicationFn) => async (request, response) => {
    const [body, port, probot] = await Promise.all([
        getBody(request),
        getPort(),
        initializeProbot(applicationFn),
    ]);
    const server = probot.server.listen(port, async () => {
        try {
            const responseFromServer = await axios_1.default({
                data: body,
                headers: request.headers,
                method: request.method,
                url: `http://localhost:${server.address().port}`,
                validateStatus: () => true,
            });
            response.statusCode = responseFromServer.status;
            Object.keys(responseFromServer.headers).forEach(headerName => {
                response.setHeader(headerName, responseFromServer.headers[headerName]);
            });
            response.end(responseFromServer.data);
        }
        catch (error) {
            response.statusCode = 500;
            response.end(error.message);
        }
        finally {
            server.close();
        }
    });
};
exports.toLambda = toLambda;
